from math import *
import numpy as np
import matplotlib.pyplot as plt
i=1j

import kagra_auxiliary_sensitivity as aux
reload(aux)

import pylab
ts=24
ls=8
params = {'axes.labelsize': ts, 'font.size': ts, 'legend.fontsize': ls, 'xtick.labelsize': ts, 'ytick.labelsize': ts, 'lines.linewidth': 2}
pylab.rcParams.update(params)

saveFlag=True
plt.close('all')

def plotdeco(xl=[5,5000],yl=[-21,-15],ttl=""):
	# DECORATIONS FOR SENSITIVITY PLOT
	plt.xlim(xl)
	plt.ylim([10**yl[0],10**yl[1]])
	plt.yticks(np.logspace(yl[0],yl[1],yl[1]-yl[0]+1))
	plt.grid(True,which='minor',linestyle=':')
	plt.grid(True,which='major',linestyle='-')
	plt.ylabel(r"displacement (${\sf /\sqrt{Hz}}$)")
	plt.xlabel("frequency (Hz)")
	if ttl!="":
		plt.title(ttl)
	plt.legend()

##### PARAMETERS
clrs={"seis":'orange',"susp":'blue',"mirr":'green',"quant":'red',"DARM":'blue','CARM':'orange',"MICH":'red',"PRCL":"green","SRCL":"purple"}

##### OPTIC
optic_list=["ITM","ETM","PRM","PR2","PR3","SRM","SR2","SR3","BS"]
OpticDict={}
for op in optic_list:
	OpticDict[op]=aux.OpticDisplacementSensitivity(op)

for op in optic_list:
	d=OpticDict[op]
	plt.figure(figsize=(8,6))
	plt.loglog(d.freq,d.total,"k-",lw=4,label="total")
	for kk in ["seis","susp","mirr"]:
		plt.loglog(d.freq,getattr(d,kk),color=clrs[kk],label=kk)
	plotdeco(ttl=op)
	plt.show()
	if saveFlag:
		plt.savefig("DisplacementNoise_%s.png" % op,bbox_inches='tight')
		data=np.transpose([d.freq,d.total,d.seis,d.susp,d.mirr])
		header="Displacement Noise of %s\nfreq total seismic suspension_thermal mirror_thermal"
		np.savetxt("DisplacementNoise_%s.txt" % op,data,delimiter=' ',fmt='%.4e',header=header)

plt.figure(figsize=(8,6))
for op in optic_list:
	d=OpticDict[op]
	plt.loglog(d.freq,d.total,"-",label=op)
	plotdeco(ttl="Mirrors")
	plt.show()
	if saveFlag:
		plt.savefig("DisplacementNoise_all.png",bbox_inches='tight')

plt.close('all')

##### DOF
DofDict={}
DofDict["DARM"]=aux.DofDisplacementSensitivity("DARM",["ITMX","ITMY","ETMX","ETMY"],[1,1,1,1])
DofDict["CARM"]=aux.DofDisplacementSensitivity("CARM",["ITMX","ITMY","ETMX","ETMY"],[1,1,1,1])
DofDict["MICH"]=aux.DofDisplacementSensitivity("MICH",["ITMX","ITMY","BS"],[1,1,sqrt(2)])
DofDict["PRCL"]=aux.DofDisplacementSensitivity("PRCL",["ITMX","ITMY","BS","PRM","PR2","PR3"],[0.5,0.5,sqrt(2)/2,1,2,2])
DofDict["SRCL"]=aux.DofDisplacementSensitivity("SRCL",["ITMX","ITMY","BS","SRM","SR2","SR3"],[0.5,0.5,sqrt(2)/2,1,2,2])

for dof in ["DARM","CARM","MICH","PRCL","SRCL"]:
	d=DofDict[dof]
	plt.figure(figsize=(8,6))
	plt.loglog(d.freq,d.total,"k-",lw=4,label="total")
	for kk in ["seis","susp","mirr","quant"]:
		plt.loglog(d.freq,getattr(d,kk),color=clrs[kk],label=kk)
	if dof=="DARM":
		data=np.transpose(np.loadtxt("spectrum_DRSE.txt"))
		plt.loglog(data[0],data[6]*3000,'--',color="magenta",label="DRSE deign")
	plotdeco(ttl=dof)
	plt.show()
	if saveFlag:
		plt.savefig("DisplacementSensitivity_%s.png" % dof,bbox_inches='tight')
		data=np.transpose([d.freq,d.total,d.seis,d.susp,d.mirr,d.quant])
		header="Displacement Sensitivity of %s\nfreq total seismic suspension_thermal mirror_thermal quantum"
		np.savetxt("DisplacementSensitivity_%s.txt" % dof,data,delimiter=' ',fmt='%.4e',header=header)

plt.figure(figsize=(8,6))
for dof in ["DARM","CARM","MICH","PRCL","SRCL"]:
	d=DofDict[dof]
	plt.loglog(d.freq,d.total,"-",color=clrs[dof],label=dof)
	plotdeco(ttl="DOFs")
plt.show()
if saveFlag:
	plt.savefig("DisplacementSensitivity_all.png",bbox_inches='tight')